// REMNANTS OF NAEZITH, GRADIENT COLORIZE SHADER
#version 110
uniform sampler2D u_texture;
uniform vec3 u_color;
uniform vec2 u_cam_pos_px;
uniform vec2 u_resolution;
uniform vec2 u_level_size_px;
uniform vec2 u_gradient_uv;
uniform float u_gradient_center;

void main(){
	// This is screen position as uv
	vec2 new_coord = vec2(gl_TexCoord[0].x, gl_TexCoord[0].y);
	
	// We grab the pixel with uv
	vec4 orig_c = texture2D(u_texture, new_coord);
	
	// Have a colorized version of the pixel as c
	vec4 c = orig_c;
	c.x = (c.x > 0.5) ? (1.0 - (1.0-2.0*(c.x-0.5)) * (1.0-u_color.x)) : ((2.0*c.x) * u_color.x);
	c.y = (c.y > 0.5) ? (1.0 - (1.0-2.0*(c.y-0.5)) * (1.0-u_color.y)) : ((2.0*c.y) * u_color.y);
	c.z = (c.z > 0.5) ? (1.0 - (1.0-2.0*(c.z-0.5)) * (1.0-u_color.z)) : ((2.0*c.z) * u_color.z);
	
	// Now we need to convert the screen uv to level-sized uv.
	new_coord.y = 1.0 - new_coord.y; // Reverse the y, because of OpenGL
	
	// Convert uv to pixels
	new_coord *= u_resolution; 
	
	// Adding the camera position to make it the world coordinate
	new_coord += u_cam_pos_px;
	
	// Divide to level size to have the level-sized uv, independant from the camera position.
	new_coord /= u_level_size_px;
	
	
	// Now we need to fade the color as gradient with the gradient direction unit vector (u_gradient_uv)
	gl_FragColor = mix(orig_c, c, 
				
				min(max(u_gradient_center + 0.5 + ((new_coord.x - 0.5) * u_gradient_uv.x +
												   (new_coord.y - 0.5) * u_gradient_uv.y)/ 2.0, 0.0), 1.0)
	);
} 
