// REMNANTS OF NAEZITH, WATER SHADER
#version 110
uniform vec3      u_color;
uniform vec2      u_resolution;
uniform vec2      u_cam_pos_uv;
uniform sampler2D u_texture;
uniform sampler2D u_texture1;
uniform float     u_refl_size_y;
uniform float     u_surface_len;
uniform vec2      u_top_left_uv;
uniform vec2      u_size_px;
uniform float     u_zoom;
uniform float     u_fade_len;
uniform vec2      u_movement1;
uniform vec2      u_movement2;

float n_step(float a, float b, float x) {
    float t = clamp((x - a) / (b - a), 0.0, 1.0);
    return t*t*(3.0 - (2.0*t));
}

void main() {
	vec2 new_coord = vec2(gl_TexCoord[0].x, gl_TexCoord[0].y); 
	
	// FIRST HEIGHTMAP
	vec2 new_map_coord1 = new_coord;
	
	// scroll the map
	new_map_coord1 += u_movement1;
	
	// clamp the texture to make it repeat indefinitely
	new_map_coord1 = fract(new_map_coord1);
	new_map_coord1.x += u_cam_pos_uv.x;
	new_map_coord1.y -= u_cam_pos_uv.y;
	
	// apply the height-map displacement to the original texture
	vec4 height_map_color1 = texture2D(u_texture1, new_map_coord1);
	new_coord.xy += (height_map_color1.rg - 0.5) * 0.005 * u_zoom;
		
	// SECOND HEIGHTMAP
	vec2 new_map_coord2 = new_coord;
	new_map_coord2 += u_movement2;
	new_map_coord2 = fract(new_map_coord2);
	new_map_coord2.x += u_cam_pos_uv.x;
	new_map_coord2.y -= u_cam_pos_uv.y;

	vec4 height_map_color2 = texture2D(u_texture1, new_map_coord2);
	new_coord.xy += (height_map_color2.rg - 0.5) * 0.005 * u_zoom;
	
	
	
	
	
	// DISTORT
	vec4 col = texture2D(u_texture, new_coord); // Coloring now...
	
	new_coord.y = 1.0 - new_coord.y; // Reverse the y, because fuck you opengl
	vec2 coord_px = new_coord - u_top_left_uv;
	coord_px.xy *= u_resolution.xy;

	
	// REFLECTION
	if(coord_px.y < u_refl_size_y) {
		vec2 real_coord = vec2(gl_TexCoord[0].x, 1.0 - gl_TexCoord[0].y);
		vec4 reflection = texture2D(u_texture, vec2(gl_TexCoord[0].x, 
			1.0 - (2.0*u_top_left_uv.y - real_coord.y))  +
			(new_coord - real_coord) 	);  // Wave the reflection
			
		reflection.a = (1.0 - n_step(0.0, u_refl_size_y, coord_px.y))*0.5; // Fade reflection and set opacity
		
		// MIX REFLECTION & DISTORT
		col.rgb = col.rgb + reflection.a*(reflection.rgb - col.rgb);
	}
	
	
	// COLORING
	col.rgb += u_color*0.125; // Adding the color
	col.rgb *= coord_px.y < u_surface_len ? 2.0 : 1.2;  // Making it brighter
	
	const float wave_thickness = 1.0;
	col.a = n_step(0.1, wave_thickness, coord_px.y) * // Cutting the top off
			n_step(-u_fade_len, -u_fade_len + wave_thickness, coord_px.x) *  // Cutting the left off
			(1.0-n_step(u_size_px.x + u_fade_len, u_size_px.x + u_fade_len + wave_thickness, coord_px.x)) * // Cutting the right off
			(1.0-n_step(u_size_px.y + u_fade_len, u_size_px.y + u_fade_len + wave_thickness, coord_px.y));  // Cutting the bottom off
			
	gl_FragColor = col; 
}